@@ -34,8 +34,9 @@
  * handles launching an application in a graphical context. It provides
  * startup notification and allows to launch applications on a specific
  * screen or workspace.
- * <example>
- * <title>Launching an application</title>
+ *
+ * ## Launching an application
+ *
  * |[<!-- language="C" -->
  * GdkAppLaunchContext *context;
  *
@@ -49,7 +50,6 @@
  *
  * g_object_unref (context);
  * ]|
- * </example>
  */
 
 
@@ -162,8 +162,10 @@ gdk_app_launch_context_get_display (GAppLaunchContext *context,
   GdkAppLaunchContext *ctx = GDK_APP_LAUNCH_CONTEXT (context);
   GdkDisplay *display;
 
+G_GNUC_BEGIN_IGNORE_DEPRECATIONS
   if (ctx->screen)
     return gdk_screen_make_display_name (ctx->screen);
+G_GNUC_END_IGNORE_DEPRECATIONS
 
   if (ctx->display)
     display = ctx->display;
@@ -236,8 +238,7 @@ gdk_app_launch_context_set_screen (GdkAppLaunchContext *context,
  * Sets the workspace on which applications will be launched when
  * using this context when running under a window manager that
  * supports multiple workspaces, as described in the
- * <ulink url="http://www.freedesktop.org/Standards/wm-spec">Extended
- * Window Manager Hints</ulink>.
+ * [Extended Window Manager Hints](http://www.freedesktop.org/Standards/wm-spec).
  *
  * When the workspace is not specified or @desktop is set to -1,
  * it is up to the window manager to pick one, typically it will
