@@ -188,8 +188,8 @@ uint32_t _gdk_wayland_device_get_implicit_grab_serial(GdkWaylandDevice *device,
 uint32_t _gdk_wayland_seat_get_last_implicit_grab_serial (GdkSeat           *seat,
                                                           GdkEventSequence **seqence);
 struct wl_data_device * gdk_wayland_device_get_data_device (GdkDevice *gdk_device);
-void gdk_wayland_device_set_selection (GdkDevice             *gdk_device,
-                                       struct wl_data_source *source);
+void gdk_wayland_seat_set_selection (GdkSeat               *seat,
+                                     struct wl_data_source *source);
 
 void gdk_wayland_seat_set_primary (GdkSeat                             *seat,
                                    struct gtk_primary_selection_source *source);
