@@ -38,9 +38,10 @@ struct _GtkBoxGadgetPrivate {
   GtkOrientation orientation;
   GArray *children;
 
-  guint draw_focus : 1;
-  guint draw_reverse : 1;
+  guint draw_focus       : 1;
+  guint draw_reverse     : 1;
   guint allocate_reverse : 1;
+  guint align_reverse    : 1;
 };
 
 typedef gboolean (* ComputeExpandFunc) (GObject *object, GtkOrientation orientation);
@@ -48,7 +49,7 @@ typedef gboolean (* ComputeExpandFunc) (GObject *object, GtkOrientation orientat
 typedef struct _GtkBoxGadgetChild GtkBoxGadgetChild;
 struct _GtkBoxGadgetChild {
   GObject *object;
-  ComputeExpandFunc compute_expand;
+  gboolean expand;
   GtkAlign align;
 };
 
@@ -64,6 +65,21 @@ gtk_box_gadget_child_is_visible (GObject *child)
     return gtk_css_gadget_get_visible (GTK_CSS_GADGET (child));
 }
 
+static gboolean
+gtk_box_gadget_child_compute_expand (GtkBoxGadget      *gadget,
+                                     GtkBoxGadgetChild *child)
+{
+  GtkBoxGadgetPrivate *priv = gtk_box_gadget_get_instance_private (GTK_BOX_GADGET (gadget));
+
+  if (child->expand)
+    return TRUE;
+
+  if (GTK_IS_WIDGET (child->object))
+    return gtk_widget_compute_expand (GTK_WIDGET (child->object), priv->orientation);
+
+  return FALSE;
+}
+
 static GtkAlign
 gtk_box_gadget_child_get_align (GtkBoxGadget      *gadget,
                                 GtkBoxGadgetChild *child)
@@ -84,6 +100,21 @@ gtk_box_gadget_child_get_align (GtkBoxGadget      *gadget,
   return align;
 }
 
+static GtkAlign
+effective_align (GtkAlign align,
+                 gboolean reverse)
+{
+  switch (align)
+    {
+    case GTK_ALIGN_START:
+      return reverse ? GTK_ALIGN_END : GTK_ALIGN_START;
+    case GTK_ALIGN_END:
+      return reverse ? GTK_ALIGN_START : GTK_ALIGN_END;
+    default:
+      return align;
+    }
+}
+
 static void
 gtk_box_gadget_measure_child (GObject        *child,
                               GtkOrientation  orientation,
@@ -132,7 +163,7 @@ gtk_box_gadget_distribute (GtkBoxGadget     *gadget,
                                     &sizes[i].minimum_size, &sizes[i].natural_size,
                                     NULL, NULL);
       if (gtk_box_gadget_child_is_visible (child->object) &&
-          child->compute_expand (child->object, priv->orientation))
+          gtk_box_gadget_child_compute_expand (gadget, child))
         n_expand++;
       size -= sizes[i].minimum_size;
     }
@@ -153,7 +184,7 @@ gtk_box_gadget_distribute (GtkBoxGadget     *gadget,
       GtkBoxGadgetChild *child = &g_array_index (priv->children, GtkBoxGadgetChild, i);
 
       if (!gtk_box_gadget_child_is_visible (child->object) ||
-          !child->compute_expand (child->object, priv->orientation))
+          !gtk_box_gadget_child_compute_expand (gadget, child))
         continue;
 
       sizes[i].minimum_size += size / n_expand;
@@ -304,6 +335,7 @@ gtk_box_gadget_allocate_child (GObject        *child,
                                              allocation->width,
                                              &minimum, &natural,
                                              &minimum_baseline, &natural_baseline);
+
           switch (child_align)
             {
             case GTK_ALIGN_FILL:
@@ -407,7 +439,7 @@ gtk_box_gadget_allocate (GtkCssGadget        *gadget,
           child_align = gtk_box_gadget_child_get_align (GTK_BOX_GADGET (gadget), child);
           gtk_box_gadget_allocate_child (child->object,
                                          priv->orientation,
-                                         child_align,
+                                         effective_align (child_align, priv->align_reverse),
                                          &child_allocation,
                                          baseline,
                                          &child_clip);
@@ -441,7 +473,7 @@ gtk_box_gadget_allocate (GtkCssGadget        *gadget,
           child_align = gtk_box_gadget_child_get_align (GTK_BOX_GADGET (gadget), child);
           gtk_box_gadget_allocate_child (child->object,
                                          priv->orientation,
-                                         child_align,
+                                         effective_align (child_align, priv->align_reverse),
                                          &child_allocation,
                                          -1,
                                          &child_clip);
@@ -595,6 +627,15 @@ gtk_box_gadget_set_allocate_reverse (GtkBoxGadget *gadget,
   priv->allocate_reverse = allocate_reverse;
 }
 
+void
+gtk_box_gadget_set_align_reverse (GtkBoxGadget *gadget,
+                                  gboolean      align_reverse)
+{
+  GtkBoxGadgetPrivate *priv = gtk_box_gadget_get_instance_private (gadget);
+
+  priv->align_reverse = align_reverse;
+}
+
 static GtkCssNode *
 get_css_node (GObject *child)
 {
@@ -605,17 +646,17 @@ get_css_node (GObject *child)
 }
 
 static void
-gtk_box_gadget_insert_object (GtkBoxGadget      *gadget,
-                              int                pos,
-                              GObject           *object,
-                              ComputeExpandFunc  compute_expand_func,
-                              GtkAlign           align)
+gtk_box_gadget_insert_object (GtkBoxGadget *gadget,
+                              int           pos,
+                              GObject      *object,
+                              gboolean      expand,
+                              GtkAlign      align)
 {
   GtkBoxGadgetPrivate *priv = gtk_box_gadget_get_instance_private (gadget);
   GtkBoxGadgetChild child;
 
   child.object = g_object_ref (object);
-  child.compute_expand = compute_expand_func;
+  child.expand = expand;
   child.align = align;
 
   if (pos < 0 || pos >= priv->children->len)
@@ -635,15 +676,11 @@ gtk_box_gadget_insert_object (GtkBoxGadget      *gadget,
 }
 
 void
-gtk_box_gadget_insert_widget (GtkBoxGadget           *gadget,
-                              int                     pos,
-                              GtkWidget              *widget)
+gtk_box_gadget_insert_widget (GtkBoxGadget *gadget,
+                              int           pos,
+                              GtkWidget    *widget)
 {
-  gtk_box_gadget_insert_object (gadget,
-                                pos,
-                                G_OBJECT (widget),
-                                (ComputeExpandFunc) gtk_widget_compute_expand,
-                                GTK_ALIGN_FILL);
+  gtk_box_gadget_insert_object (gadget, pos, G_OBJECT (widget), FALSE, GTK_ALIGN_FILL);
 }
 
 static GtkBoxGadgetChild *
@@ -692,34 +729,11 @@ gtk_box_gadget_remove_widget (GtkBoxGadget *gadget,
   gtk_box_gadget_remove_object (gadget, G_OBJECT (widget));
 }
 
-static gboolean
-only_horizontal (GObject        *object,
-                 GtkOrientation  orientation)
-{
-  return orientation == GTK_ORIENTATION_HORIZONTAL;
-}
-
-static gboolean
-only_vertical (GObject        *object,
-               GtkOrientation  orientation)
-{
-  return orientation == GTK_ORIENTATION_VERTICAL;
-}
-
-static ComputeExpandFunc
-expand_func_from_flags (gboolean hexpand,
-                        gboolean vexpand)
-{
-  return hexpand ? (vexpand ? (ComputeExpandFunc) gtk_true : only_horizontal)
-                 : (vexpand ? only_vertical : (ComputeExpandFunc) gtk_false);
-}
-
 void
 gtk_box_gadget_insert_gadget_before (GtkBoxGadget *gadget,
                                      GtkCssGadget *sibling,
                                      GtkCssGadget *cssgadget,
-                                     gboolean      hexpand,
-                                     gboolean      vexpand,
+                                     gboolean      expand,
                                      GtkAlign      align)
 {
   /* Insert at the end if no sibling specified */
@@ -728,15 +742,14 @@ gtk_box_gadget_insert_gadget_before (GtkBoxGadget *gadget,
   if (sibling)
     gtk_box_gadget_find_object (gadget, G_OBJECT (sibling), &pos);
 
-  gtk_box_gadget_insert_gadget (gadget, pos, cssgadget, hexpand, vexpand, align);
+  gtk_box_gadget_insert_gadget (gadget, pos, cssgadget, expand, align);
 }
 
 void
 gtk_box_gadget_insert_gadget_after (GtkBoxGadget *gadget,
                                     GtkCssGadget *sibling,
                                     GtkCssGadget *cssgadget,
-                                    gboolean      hexpand,
-                                    gboolean      vexpand,
+                                    gboolean      expand,
                                     GtkAlign      align)
 {
   /* Insert at the beginning if no sibling specified */
@@ -745,25 +758,17 @@ gtk_box_gadget_insert_gadget_after (GtkBoxGadget *gadget,
   if (sibling && gtk_box_gadget_find_object (gadget, G_OBJECT (sibling), &pos))
     pos++;
 
-  gtk_box_gadget_insert_gadget (gadget, pos, cssgadget, hexpand, vexpand, align);
+  gtk_box_gadget_insert_gadget (gadget, pos, cssgadget, expand, align);
 }
 
 void
 gtk_box_gadget_insert_gadget (GtkBoxGadget *gadget,
                               int           pos,
                               GtkCssGadget *cssgadget,
-                              gboolean      hexpand,
-                              gboolean      vexpand,
+                              gboolean      expand,
                               GtkAlign      align)
 {
-  ComputeExpandFunc func;
-
-  func = expand_func_from_flags (hexpand, vexpand);
-  gtk_box_gadget_insert_object (gadget,
-                                pos,
-                                G_OBJECT (cssgadget),
-                                func,
-                                align);
+  gtk_box_gadget_insert_object (gadget, pos, G_OBJECT (cssgadget), expand, align);
 }
 
 void
@@ -795,34 +800,31 @@ gtk_box_gadget_reverse_children (GtkBoxGadget *gadget)
 
 void
 gtk_box_gadget_set_gadget_expand (GtkBoxGadget *gadget,
-                                  GtkCssGadget *cssgadget,
-                                  gboolean      hexpand,
-                                  gboolean      vexpand)
+                                  GObject      *object,
+                                  gboolean      expand)
 {
   GtkBoxGadgetChild *child;
-  ComputeExpandFunc func;
 
-  child = gtk_box_gadget_find_object (gadget, G_OBJECT (cssgadget), NULL);
+  child = gtk_box_gadget_find_object (gadget, object, NULL);
 
   if (!child)
     return;
 
-  func = expand_func_from_flags (hexpand, vexpand);
-  if (child->compute_expand == func)
+  if (child->expand == expand)
     return;
 
-  child->compute_expand = func;
+  child->expand = expand;
   gtk_css_gadget_queue_resize (GTK_CSS_GADGET (gadget));
 }
 
 void
 gtk_box_gadget_set_gadget_align (GtkBoxGadget *gadget,
-                                 GtkCssGadget *cssgadget,
+                                 GObject      *object,
                                  GtkAlign      align)
 {
   GtkBoxGadgetChild *child;
 
-  child = gtk_box_gadget_find_object (gadget, G_OBJECT (cssgadget), NULL);
+  child = gtk_box_gadget_find_object (gadget, object, NULL);
 
   if (!child)
     return;
