@@ -26,6 +26,7 @@
 #include <gdk/gdk.h>
 
 #include "gtksearchenginesimple.h"
+#include "gtkfilesystem.h"
 #include "gtkprivate.h"
 
 #include <string.h>
@@ -87,26 +88,30 @@ gtk_search_engine_simple_dispose (GObject *object)
   G_OBJECT_CLASS (_gtk_search_engine_simple_parent_class)->dispose (object);
 }
 
+static void
+queue_if_local (SearchThreadData *data,
+                GFile            *file)
+{
+  if (file &&
+      !_gtk_file_consider_as_remote (file) &&
+      !g_file_has_uri_scheme (file, "recent"))
+    g_queue_push_tail (data->directories, g_object_ref (file));
+}
+
 static SearchThreadData *
 search_thread_data_new (GtkSearchEngineSimple *engine,
 			GtkQuery              *query)
 {
   SearchThreadData *data;
-  const gchar *uri;
-  GFile *location;
 
   data = g_new0 (SearchThreadData, 1);
 
   data->engine = g_object_ref (engine);
   data->directories = g_queue_new ();
   data->query = g_object_ref (query);
-  data->recursive = _gtk_search_engine_get_recursive (GTK_SEARCH_ENGINE (engine));
-  uri = gtk_query_get_location (query);
-  if (uri != NULL)
-    location = g_file_new_for_uri (uri);
-  else
-    location = g_file_new_for_path (g_get_home_dir ());
-  g_queue_push_tail (data->directories, location);
+  /* Simple search engine is too slow to be recursive */
+  data->recursive = FALSE;
+  queue_if_local (data, gtk_query_get_location (query));
 
   data->cancellable = g_cancellable_new ();
 
@@ -246,7 +251,7 @@ visit_directory (GFile *dir, SearchThreadData *data)
           GtkSearchHit *hit;
 
           hit = g_new (GtkSearchHit, 1);
-          hit->uri = g_file_get_uri (child);
+          hit->file = g_object_ref (child);
           hit->info = g_object_ref (info);
           data->hits = g_list_prepend (data->hits, hit);
         }
@@ -258,7 +263,7 @@ visit_directory (GFile *dir, SearchThreadData *data)
       if (data->recursive &&
           g_file_info_get_file_type (info) == G_FILE_TYPE_DIRECTORY &&
           !is_indexed (data->engine, child))
-        g_queue_push_tail (data->directories, g_object_ref (child));
+        queue_if_local (data, child);
     }
 
   g_object_unref (enumerator);
